/*
 * Copyright (c) 2012, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <avr/io.h>
#include <stdint.h>
#include <stdbool.h>

#include "curve.h"
#include "eeprom.h"
#include "pwm.h"
#include "usart.h"
#include "proto.h"

void recv_frame(struct frame *f)
{
	f->type = usart_rx_byte();
	
	for (uint8_t n = 0; n < MAX_PARAM_LEN; n++)
		f->param[n] = usart_rx_byte();
}

void send_frame(struct frame *f)
{
	usart_tx_byte(f->type);
	
	for (uint8_t n = 0; n < MAX_PARAM_LEN; n++)
		usart_tx_byte(f->param[n]);
}

bool process_set_pwm(const void *in)
{
	const struct param_set_pwm *p = in;
	bool result = false;

	switch (p->chan) {
	case PARAM_CHAN_A:
		pwm_set_dc(PWM_CHANNEL_A, p->pwm);
		result = true;
		break;
	case PARAM_CHAN_B:
		pwm_set_dc(PWM_CHANNEL_B, p->pwm);
		result = true;
		break;
	}

	return result;
}

bool process_set_volt(const void *in)
{
	const struct param_set_volt *p = in;
	uint8_t pwm;
	bool result = false;

	switch (p->chan) {
	case PARAM_CHAN_A:
		pwm = curve_16_8(ee_curve_1_x, ee_curve_1_y,
				EE_CURVE_DATA_POINTS, p->volt);
		pwm_set_dc(PWM_CHANNEL_A, pwm);
		result = true;
		break;
	case PARAM_CHAN_B:
		pwm = curve_16_8(ee_curve_2_x, ee_curve_2_y,
				EE_CURVE_DATA_POINTS, p->volt);
		pwm_set_dc(PWM_CHANNEL_B, pwm);
		result = true;
		break;
	}

	return result;
}

bool process_set_data(const void *in)
{
	const struct param_set_data *p = in;
	bool result = false;

	switch (p->chan) {
	case PARAM_CHAN_A:
		if (p->index < EE_CURVE_DATA_POINTS) {
			ee_curve_1_x[p->index] = p->xval;
			ee_curve_1_y[p->index] = p->yval;
			result = true;
		}
		break;
	case PARAM_CHAN_B:
		if (p->index < EE_CURVE_DATA_POINTS) {
			ee_curve_2_x[p->index] = p->xval;
			ee_curve_2_y[p->index] = p->yval;
			result = true;
		}
		break;
	}

	return result;
}

bool process_get_data(const void *in, void *out)
{
	const struct param_get_data *p = in;
	struct result_get_data *r = out;
	bool result = false;

	switch (p->chan) {
	case PARAM_CHAN_A:
		if (p->index < EE_CURVE_DATA_POINTS) {
			r->xval = ee_curve_1_x[p->index];
			r->yval = ee_curve_1_y[p->index];
			result = true;
		}
		break;
	case PARAM_CHAN_B:
		if (p->index < EE_CURVE_DATA_POINTS) {
			r->xval = ee_curve_2_x[p->index];
			r->yval = ee_curve_2_y[p->index];
			result = true;
		}
		break;
	}

	return result;
}

void process_frame(const struct frame *req, struct frame *res)
{
	bool result = false;

	switch (req->type) {
	case CMD_SET_PWM:
		result = process_set_pwm(req->param);
		break;
	case CMD_SET_VOLT:
		result = process_set_volt(req->param);
		break;
	case CMD_SET_DATA:
		result = process_set_data(req->param);
		break;
	case CMD_GET_DATA:
		result = process_get_data(req->param, res->param);
		break;
	case CMD_LOAD_DATA:
		ee_load_data();
		result = true;
		break;
	case CMD_STORE_DATA:
		ee_store_data();
		result = true;
		break;
	}

	res->type = result ? RES_POSITIVE : RES_NEGATIVE;
}

int main(void)
{
	struct frame req;
	struct frame res;

	pwm_init();
	usart_init();
	ee_load_data();

	for (;;) {
		recv_frame(&req);
		process_frame(&req, &res);
		send_frame(&res);
	}
}

